@@ -17,10 +17,10 @@
  *
  * The Initial Developer of the Original Code is
  * Mozilla Foundation.
- * Portions created by the Initial Developer are Copyright (C) 2008
+ * Portions created by the Initial Developer are Copyright (C) 2006
  * the Initial Developer. All Rights Reserved.
  *
- * Contributor(s): Jesse Ruderman
+ * Contributor(s): Igor Bukanov
  *
  * Alternatively, the contents of this file may be used under the terms of
  * either the GNU General Public License Version 2 or later (the "GPL"), or
@@ -36,41 +36,28 @@
  *
  * ***** END LICENSE BLOCK ***** */
 
-gTestfile = 'regress-458679-02.js';
+gTestfile = 'regress-356238-02.js';
 
-var summary = 'GetXMLEntity should not assume FastAppendChar is infallible';
-var BUGNUMBER = 458679;
-var actual = '';
-var expect = '';
+var BUGNUMBER = 356238;
+var summary = 'bug 356238';
+var actual = 'No Duplicate';
+var expect = 'No Duplicate';
 
 printBugNumber(BUGNUMBER);
 START(summary);
 
-function stringOfLength(n)
-{
-    if (n == 0) {
-        return "";
-    } else if (n == 1) {
-        return "<";
-    } else {
-        var r = n % 2;
-        var d = (n - r) / 2;
-        var y = stringOfLength(d);
-        return y + y + stringOfLength(r);
-    }    
+var xml = <xml><child><a/><b/></child></xml>;
+var child = xml.child[0];
+
+try {
+    child.insertChildAfter(null, xml.child);
+    actual = "insertChildAfter succeeded when it should throw an exception";
+}
+catch (e) {
 }
 
-try
-{
-
-    void stringOfLength(4435455);
-    x = stringOfLength(14435455);
-    <xxx>{x}</xxx>;
-}
-catch(ex)
-{
-}
+if (child.a[0] === child.a[1])
+    actual = 'Duplicate detected';
 
 TEST(1, expect, actual);
-
 END();
