@@ -19,7 +19,7 @@
  * Portions created by the Initial Developer are Copyright (C) 2005
  * the Initial Developer. All Rights Reserved.
  *
- * Contributor(s): moz_bug_r_a4
+ * Contributor(s): Igor Bukanov
  *
  * Alternatively, the contents of this file may be used under the terms of
  * either the GNU General Public License Version 2 or later (the "GPL"), or
@@ -35,27 +35,16 @@
  *
  * ***** END LICENSE BLOCK ***** */
 
-var gTestfile = 'regress-315509-01.js';
+var gTestfile = 'regress-310425-02.js';
 //-----------------------------------------------------------------------------
-var BUGNUMBER = 315509;
-var summary = 'Array.prototype.unshift on Arrays with holes';
+var BUGNUMBER = 310425;
+var summary = 'Array.indexOf/lastIndexOf edge cases';
 var actual = '';
 var expect = '';
 
 printBugNumber(BUGNUMBER);
 printStatus (summary);
-
-var a = [0,1,2,3,4];
-delete a[1];
-
-expect = '0,,2,3,4';
-actual = a.toString();
-
-reportCompare(expect, actual, summary);
-
-a.unshift('a','b');
-
-expect = 'a,b,0,,2,3,4';
-actual = a.toString();
  
+expect = -1;
+actual = Array(1).indexOf(1);
 reportCompare(expect, actual, summary);
