@@ -19,7 +19,7 @@
  * Portions created by the Initial Developer are Copyright (C) 2008
  * the Initial Developer. All Rights Reserved.
  *
- * Contributor(s): Jesse Ruderman
+ * Contributor(s):
  *
  * Alternatively, the contents of this file may be used under the terms of
  * either the GNU General Public License Version 2 or later (the "GPL"), or
@@ -35,10 +35,10 @@
  *
  * ***** END LICENSE BLOCK ***** */
 
-var gTestfile = 'regress-465460-11.js';
+var gTestfile = 'regress-452498-099.js';
 //-----------------------------------------------------------------------------
-var BUGNUMBER = 465460;
-var summary = 'TM: valueOf in a loop: do not assert';
+var BUGNUMBER = 452498;
+var summary = 'TM: upvar2 regression tests';
 var actual = '';
 var expect = '';
 
@@ -52,12 +52,34 @@ function test()
   enterFunc ('test');
   printBugNumber(BUGNUMBER);
   printStatus (summary);
- 
-  jit(true);
 
-  for (let d=0;d<2;++d) for (let a=0;a<1;++a) "" + (d && function(){});
+// ------- Comment #99 From Gary Kwong [:nth10sd]
 
-  jit(false);
+  try
+  {
+    eval("(function x(){x.(this)} )();");
+  }
+  catch(ex)
+  {
+  }
+
+// Assertion failure: (uint32)(index_) < atoms_->length, at ../jsinterp.cpp:327
+// Crash [@ js_FullTestPropertyCache] at null in opt, -j not required.
+
+// =====
+
+  try
+  {
+    (function(){try {x} finally {}; ([x in []] for each (x in x))})();
+  }
+  catch(ex)
+  {
+  }
+
+// Assertion failure: lexdep->frameLevel() <= funbox->level, at ../jsparse.cpp:1735
+// Crash [@ BindNameToSlot] near null in opt, -j not required.
+
+// =====
 
   reportCompare(expect, actual, summary);
 
