@@ -1423,15 +1423,15 @@ pkix_pl_HttpDefaultClient_TrySendAndReceive(
 
                         /* PR_smprintf_free original header buffer */
                         PR_smprintf_free(sendbuf);
+                        sendbuf = NULL;
                         
                 } else if (client->send_http_method == HTTP_GET_METHOD) {
-                        sendbuf = PR_smprintf
+                        client->GETBuf = PR_smprintf
                             ("GET %s HTTP/1.1\r\nHost: %s:%d\r\n\r\n",
                             client->path,
                             client->host,
                             client->portnum);
-                        client->GETBuf = sendbuf;
-                        client->GETLen = PORT_Strlen(sendbuf);
+                        client->GETLen = PORT_Strlen(client->GETBuf);
                 }
 
         }
@@ -1497,6 +1497,9 @@ pkix_pl_HttpDefaultClient_TrySendAndReceive(
         }
 
 cleanup:
+        if (sendbuf) {
+            PR_smprintf_free(sendbuf);
+        }
 
         PKIX_RETURN(HTTPDEFAULTCLIENT);
 
